﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace SPImageViewer
{
    public partial class Form1 : Form
    {
        FileInfo[] imageFiles;
        int imageIndex;

        public Form1()
        {
            InitializeComponent();
        }

        //「終了」メニュー
        private void CloseMenuItem_Click(object sender, EventArgs e)
        {
            Close();
        }

        //「フォルダを選択」メニュー
        private void ChooseFolderMenuItem_Click(object sender, EventArgs e)
        {
            DialogResult dr = folderBrowserDialog1.ShowDialog();

            if (dr == DialogResult.OK)
            {
                DirectoryInfo di = new DirectoryInfo(folderBrowserDialog1.SelectedPath);                
                FileInfo[] fi = di.GetFiles();
                int count = 0;
                
                if (fi.Length == 0)
                {
                    MessageBox.Show("画像ファイルが見つかりませんでした");
                    return;
                }

                while (true)
                {
                    if (count + 1 == fi.Length)
                    {
                        MessageBox.Show("画像ファイルが見つかりませんでした");                        
                        return;
                    }
                    if (IsImageFile(fi[count].FullName))
                        break;
                    count++;
                }
                
                imageFiles = fi;
                imageIndex = count;
                pictureBox1.Image = Image.FromFile(imageFiles[imageIndex].FullName);                
            }
        }

        //キーが押されたときに呼ばれる
        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            if (imageFiles == null || imageFiles.Length == 0)
                return;
            
            if (e.KeyData == Keys.Right)
            {   
                string fileName = GetNextFileName();
                pictureBox1.Image.Dispose();
                pictureBox1.Image = Image.FromFile(fileName);
                ChangeSizeMode();
            }
            else if (e.KeyData == Keys.Left)
            {                
                string fileName = GetPreviewFileName();
                pictureBox1.Image.Dispose();
                pictureBox1.Image = Image.FromFile(fileName);
                ChangeSizeMode();
            }
        }

        //次に表示するべき”画像ファイル名”を返す
        private string GetNextFileName()
        {
            while (true)
            {
                if (imageIndex + 1 >= imageFiles.Length)
                    imageIndex = -1;

                imageIndex++;
                if (IsImageFile(imageFiles[imageIndex].FullName))
                    return imageFiles[imageIndex].FullName;
            }
        }

        //引数のパスのファイルが画像ファイルかどうかを判定
        private bool IsImageFile(string filePath)
        {
            string extension = Path.GetExtension(filePath);
            switch (extension) { 
                case ".jpg":
                    return true;
                case ".png":
                    return true;
                case ".bmp":
                    return true;
                case ".gif":
                    return true;
                default:
                    return false;
            }
        }

        //一つ前の”画像ファイル名”を返す
        private string GetPreviewFileName()
        {
            while (true)
            {
                if (imageIndex == 0)
                    imageIndex = imageFiles.Length;

                imageIndex--;
                if (IsImageFile(imageFiles[imageIndex].FullName))
                    return imageFiles[imageIndex].FullName;
            }
        }

        //画像の大きさとピクチャーボックスコントロールの大きさを比較して、その結果によってサイズモードを変更する
        private void ChangeSizeMode()
        {
            if (pictureBox1.Image == null)
                return;

            if (pictureBox1.Image.Width <= pictureBox1.Width && pictureBox1.Image.Height <= pictureBox1.Height)
                pictureBox1.SizeMode = PictureBoxSizeMode.CenterImage;
            else
                pictureBox1.SizeMode = PictureBoxSizeMode.Zoom;
        }

        //フォームの大きさが変更されるときに呼ばれる
        private void Form1_Resize(object sender, EventArgs e)
        {
            ChangeSizeMode();
        }

        //「右回りに回転」メニュー
        private void RotateRightMenuItem_Click(object sender, EventArgs e)
        {
            if (pictureBox1.Image != null)
            {
                pictureBox1.Image.RotateFlip(RotateFlipType.Rotate90FlipNone);
                pictureBox1.Invalidate();
            }
        }

        //「左回りに回転」メニュー
        private void RotateLeftMenuItem_Click(object sender, EventArgs e)
        {
            if (pictureBox1.Image != null)
            {
                pictureBox1.Image.RotateFlip(RotateFlipType.Rotate90FlipXY);
                pictureBox1.Invalidate();
            }
        }

        //ピクチャーボックス上でマウスが動いたときに呼ばれる
        private void pictureBox1_MouseMove(object sender, MouseEventArgs e)
        {
            if (e.Y <= 80)
                menuStrip1.Visible = true;
            else
                menuStrip1.Visible = false;
        }

        //「外枠を非表示にする」メニュー
        private void NoBorderMenuItem_Click(object sender, EventArgs e)
        {
            if (pictureBox1.Image == null)
            {
                NoBorderMenuItem.Checked = false;
                return;
            }

            if (this.FormBorderStyle == FormBorderStyle.Sizable)
            {
                this.FormBorderStyle = FormBorderStyle.None;
                this.TransparencyKey = this.BackColor;
            }
            else
            {
                this.FormBorderStyle = FormBorderStyle.Sizable;
                this.TransparencyKey = Color.Empty;
            }
        }
                
    }
}
